package handelers

import (
	"encoding/json"
	"fmt"
	"math"
	"net/http"
	"strconv"
	"strings"
)

// ISO 3166-1 alpha-2 to alpha-3 countrycodes
var isoAlpha2ToAlpha3 = make(map[string]string)

// Henter befolkningsdata fra eksternt API
func fetchPopulationData() (*PopulationResponse, error) {
	url := "http://129.241.150.113:3500/api/v0.1/countries/population"
	resp, err := http.Get(url)
	if err != nil {
		return nil, err
	}
	defer resp.Body.Close()

	var data PopulationResponse
	err = json.NewDecoder(resp.Body).Decode(&data)
	if err != nil {
		return nil, err
	}

	// Bygg dynamisk oppslagskart for landkoder
	for _, country := range data.Data {
		if len(country.Code) >= 2 {
			isoAlpha2ToAlpha3[strings.ToUpper(country.Code[:2])] = country.Code // F.eks. NO → NOR
		}
	}

	return &data, nil
}

func getPopulationForCountry(alpha2Code string, startYear, endYear int) (*PopulationAPIResponse, error) {
	data, err := fetchPopulationData()
	if err != nil {
		return nil, err
	}

	// Konverter ISO 3166-1 alpha-2 til alpha-3
	alpha3Code, exists := isoAlpha2ToAlpha3[strings.ToUpper(alpha2Code)]
	if !exists {
		return nil, fmt.Errorf(
			ERROR_RETRIEVING_COUNTRY_DATA +
			COUNTRY_CODE_REMINDER+
			YEAR_LIMIT_REMINDER+
			EXAMPLE_URL +
			ASSISTANCE_REMINDER,
			alpha2Code,POPULATION_URL_EXAMPLE, HOMEPAGE_DEFAULT)
	}

	// Finn landet basert på ISO 3166-1 alpha-3 kode
	var countryData *Country
	for _, c := range data.Data {
		if c.Code == alpha3Code {
			countryData = &c
			break
		}
	}

	var filteredValues []PopulationYears
	var total int
	count := 0

	for _, stk := range countryData.PopulationCounts {
		if (startYear == 0 || stk.Year >= startYear) && (endYear == 0 || stk.Year <= endYear) {
			filteredValues = append(filteredValues, stk)
			total += stk.Value
			count++
		}
	}

	mean := 0
	if count > 0 {
		mean = int(math.Round(float64(total) / float64(count)))
	}

	return &PopulationAPIResponse{
		Name:         countryData.Name,
		Gjennomsnitt: mean,
		Values:       filteredValues,
	}, nil
}

func PopulationHandler(w http.ResponseWriter, r *http.Request) {
	path := strings.TrimPrefix(r.URL.Path, POPULATION_DEFAULT)
	countryCode := strings.ToUpper(path) // ISO 3166-2

	query := r.URL.RawQuery
	limit := strings.Replace(query, "limit=", "", 1)

	startYear, endYear := 0, 0

	if len(limit) > 0 {
		yearRange := strings.Split(limit, "-")
		if len(yearRange) == 2 {
			startYear, _ = strconv.Atoi(yearRange[0])
			endYear, _ = strconv.Atoi(yearRange[1])
		} else {
			fmt.Println("Year range format is incorrect")
		}
	}

	result, err := getPopulationForCountry(countryCode, startYear, endYear)
	if err != nil {
		http.Error(w, err.Error(), http.StatusNotFound)
		return
	}

	w.Header().Set(HEADER_CONTENT_TYPE, FORMAT_JSON)
	json.NewEncoder(w).Encode(result)
}
